package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.config.Config;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.billing.AutoRechargeEnum;
import com.gic.enterprise.constant.billing.AutoRechargeRedisConstant;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.redis.data.util.RedisUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("billingAccountApiService")
public class BillingAccountApiServiceImpl implements BillingAccountApiService {
    private static final Logger LOGGER = LoggerFactory.getLogger(BillingAccountApiServiceImpl.class);
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private PackageApiService packageApiService;
    @Autowired
    private BillingEnterpriseSmsPackageService billingEnterpriseSmsPackageService;
    @Autowired
    private BillingCouponCardService billingCouponCardService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private TodoItemApiService todoItemApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private Config config;

    @Override
    public ServiceResponse<BillingAccountDTO> getByEnterpriseId(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        TabBillingAccount account = billingAccountService.getByEnterpriseId(enterpriseId);
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该商户没有账户信息");
        }
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingAccountDTO.class, account));
    }

    @Override
    public ServiceResponse<Void> closeAutoRechargeConfig(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        TabBillingAccount account = billingAccountService.getByEnterpriseId(enterpriseId);
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该商户没有账户信息");
        }
        account.setAutoRecharge(AutoRechargeEnum.NO_AUTO.getCode());
        billingAccountService.updateBillingAccountByEnterpriseId(account);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> setAutoRecharge(String smsCode, Integer preNumber, Integer enterpriseId) {
        SmsPackageDTO smsPackageDTO = getSmsPackageDict(smsCode);
        if (smsPackageDTO == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "套餐包选择有误");
        }
        if (preNumber == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "自动续订条件有误");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID为空");
        }
        String key = "enterprise:billing:account:auto-recharge-set:"
                + enterpriseId;
        RedisUtil.delCache(key);
        Map<String, Object> resultMap = new HashMap<>(4);
        resultMap.put("1", preNumber.intValue());
        resultMap.put("2", smsPackageDTO);
        RedisUtil.setCache(key, resultMap);

        TabBillingAccount record = new TabBillingAccount();
        record.setEnterpriseId(enterpriseId);
        record.setAutoRecharge(AutoRechargeEnum.AUTO.getCode());
        billingAccountService.updateBillingAccountByEnterpriseId(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Map<String, Object>> getAutoRechargeConfig(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID有误");
        }
        TabBillingAccount account = billingAccountService.getByEnterpriseId(enterpriseId);
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该商户没有账户信息");
        }
        if (account.getAutoRecharge() == AutoRechargeEnum.AUTO.getCode()) {
            String key = "enterprise:billing:account:auto-recharge-set:" + enterpriseId;
            Map<String, Object> resultMap = (Map<String, Object>) RedisUtil.getCache(key);
            if (resultMap == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
            return ServiceResponse.success(resultMap);
        } else {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "自动续订短信套餐包配置已关闭");
        }
    }

    @Override
    public ServiceResponse<Void> rechargeSmsPackageTimingTask(String param) {
        //查询所有的商户
        List<EnterpriseDTO> enterpriseDTOList = enterpriseService.listEnterprise();
        for (EnterpriseDTO enterpriseDTO : enterpriseDTOList) {
            Integer enterpriseId = enterpriseDTO.getEnterpriseId();
            dealWithRechargeAuto(enterpriseId);
        }
        this.billingCouponCardService.expireCoupCard();
        return ServiceResponse.success();
    }

    private void dealWithRechargeAuto(Integer enterpriseId) {
        TabBillingAccount account = billingAccountService.getByEnterpriseId(enterpriseId);
        if (account == null) {
            LOGGER.info("{},没有账户设置", enterpriseId);
            return;
        }
        //自动续订开启并且余额大于0
        boolean isPass = account.getAutoRecharge() == AutoRechargeEnum.AUTO.getCode()
                && account.getAccountBalance() > 0;
        if (isPass) {
            String key = "enterprise:billing:account:auto-recharge-set:" + enterpriseId;
            Map<String, Object> resultMap = (Map<String, Object>) RedisUtil.getCache(key);
            if (resultMap == null) {
                LOGGER.info("{}，redis没有自动续订配置数据", enterpriseId);
            } else {
                SmsPackageDTO smsPackageDTO = (SmsPackageDTO) resultMap
                        .get(AutoRechargeRedisConstant.SMS_PACKAGE_MAP_KEY);
                //短信套餐包需要扣款金额
                Double money = smsPackageDTO.getItemCount() * smsPackageDTO.getItemFee();
                //自动续订余额前提数量
                Integer preNumber = (Integer) resultMap.get(AutoRechargeRedisConstant.PRE_NUMBER_MAP_KEY);
                if (preNumber > getSmsBalanceNumber(enterpriseId)) {
                    //进行自动扣款
                    account.setAccountBalance(account.getAccountBalance() - money);
                    billingAccountService.updateBillingAccount(account);
                }
            }
        } else {
            LOGGER.info("{},自动续订关系或者余额小于等于0", enterpriseId);
        }
    }

    /**
     * 获取短信套餐包余额数量
     * @Title: getSmsBalanceNumber 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  java.lang.Integer 
      */
    private Integer getSmsBalanceNumber(Integer enterpriseId) {
        List<BillingEnterpriseSmsPackageDTO> list = billingEnterpriseSmsPackageService.listSmsPackage(enterpriseId);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String today = format.format(new Date());
        Integer smsBalance = 0;
        for (BillingEnterpriseSmsPackageDTO smsPackageDTO : list) {
            String validDate = format.format(smsPackageDTO.getEffectiveDate());
            //是否过期
            boolean isExpired = validDate.compareTo(today) < 0;
            if (!isExpired) {
                smsBalance += smsPackageDTO.getRemainAccount();
            }
        }
        return smsBalance;
    }

    @Override
    public ServiceResponse<Void> updateAccountAfterRecharge(double totalFee, Integer enterpriseId) {
        billingAccountService.updateAccountAfterRechargeOrRefund(totalFee, enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> deductAccount(double totalFee, Integer enterpriseId) {
        int i = billingAccountService.deductAccount(totalFee, enterpriseId);
        LOGGER.info("扣费是否成功:{}", i > 0);
        if (i > 0) {
            TabBillingAccount account = this.billingAccountService.getByEnterpriseId(enterpriseId);
            if(account.getAccountBalance() <= 0){
                String content = "您的GIC商户余额已欠费"+(-account.getAccountBalance())+"元，为了不影响业务的开展，请及时<a href='"+config.getHost()+"cost-center/billing-center/recharge'>充值</a>";
                this.pushMessageApiService.pushMessage(enterpriseId, "产品通知", "商户欠费", content);

                // 待办事项
                sendTotoItem(enterpriseId);
            }
            return ServiceResponse.success(i);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED);
        }
    }

    private void sendTotoItem(Integer enterpriseId) {
        TodoItemDTO todoItemDTO = new TodoItemDTO();
        UserDTO userDTO = userApiService.getUserByEnterpriseId(enterpriseId).getResult();
        // 商户
        todoItemDTO.setEnterpriseId(enterpriseId);
        // 待办事项的userId
        todoItemDTO.setUserId(userDTO.getUserId());
        // 事项详情
        todoItemDTO.setItemContent("您的GIC商户余额已欠费");
        // 事项code 在运维后台字典配置 类目:待办事项类型
        todoItemDTO.setItemType(Constants.TODO_ITEM_ACCOUNT_BALANCE);
        // 跳转地址 待办事项点击后要跳转的地址
        todoItemDTO.setItemInfoUrl(Constants.TODO_ITEM_ACCOUNT_BALANCE_URL);
        todoItemApiService.saveTodoItem(todoItemDTO);
    }

    private SmsPackageDTO getSmsPackageDict(String smsCode) {
        ServiceResponse<List<SmsPackageDTO>> list = packageApiService.listAllSmsPackage();
        if (list.isSuccess()) {
            for (SmsPackageDTO dto : list.getResult()) {
                if (dto.getSmsPackageCode().equals(smsCode)) {
                    return dto;
                }
            }
        }
        return null;
    }

}
